/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                       *
 *                                                                         *
 ***************************************************************************/
#ifndef MUSICDATABASE_H
#define MUSICDATABASE_H

#include <qmap.h>
#include <qstring.h>
#include <qobject.h>
#include <qmutex.h>


class Song;
class FileScanThread;
class DBDir;


typedef QMap<QString,DBDir*>::iterator DBDirItr;
typedef QMap<QString,Song*>::iterator SongItr;

/**
@author Joris Guisson
@brief Keeps track of all Song's

This class keeps track of all songs in the user's music collection.
It also keeps a list of directories. The Songs are stored in a map,
using the Song's filename as key.
*/
class MusicDatabase : public QObject
{
	Q_OBJECT
	
	QMap<QString,DBDir*> m_db_dirs;
	QMap<QString,Song*> m_files;
	
	
	FileScanThread* m_fscan;
	
	/**
	Add a directory to the database.
	This should only be used by FileScanThread.
	@param d The DBDir
	*/
	void addDir(DBDir* d);
	
	friend class FileScanThread;
public:
	MusicDatabase(QObject* parent = 0,const char* name = 0);
	virtual ~MusicDatabase();

	/**
		Add a Song, to the database.
		@param s The Song
	 * @param scan Scan or not
	*/
	void addSong(Song* s,bool scan = true);
	
	/**
		Remove a Song from the database.
		@param fn The Song's filename
	*/
	void removeSong(const QString & fn);
	
	/**
		Find a Song.
		@param fn The Song's filename
		@return The Song, or 0 if it can't be found
	*/
	Song* getSong(const QString & fn);
	
	/**
	 * Rename a Song in the database, not on disk.
	 * Does nothing if fn is not in the database.
	 * @param fn The old filename
	 * @param new_name The new filename
	 * @return The Song which was changed, or 0 if fn isn't in the database
	 */
	Song* renameSong(const QString & fn,const QString & new_name);
	
	/**
		Get the number of songs.
	*/
	unsigned int getNumSongs() const {return m_files.size();}
	
	/**
		Remove all songs from the DB.
	*/
	void clear();
	
	/**
		Get an iterator to the beginnng of the map of songs.
	*/
	SongItr beginSongs() {return m_files.begin();}
	
	/**
		Get an iterator to the ending of the map of songs.
	*/
	SongItr endSongs() {return m_files.end();}
	
	/**
		Add a directory to the database. This
		will start scaning the directory.
		@param dir The name of the dir
	*/
	void addDir(const QString & dir); 
	
	/**
	 * Rename a directory in the database, not on disk.
	 * @param old_name The old name
	 * @param new_name The new name
	 */
	void renameDir(const QString & old_name,const QString & new_name);
	
	/// Get the number of dirs.
	unsigned int getNumDirs() const {return m_db_dirs.size();}
	
	/// Get an iterator to the beginning of the list of directories.
	DBDirItr beginDirs() {return m_db_dirs.begin();}
	
	/// Get an iterator to the ending of the list of directories.
	DBDirItr endDirs() {return m_db_dirs.end();}
	
	/**
		Remove a dir from the database. This will also
		remove all the songs and child dirs which are located
		in the directory.
		@param dir The directory
	*/
	void removeDir(const QString & dir);
	
	/**
		Save the database.
		@param file The file
	*/
	void saveDB(const QString & file);
	
	/**
		Load the database.
		@param file The file
	*/
	void loadDB(const QString & file);
	
signals:
	/**
		Signal emitted when a Song is added.
		@param s The Song
	*/
	void songAdded(Song* s);
	
	/**
		Signal emitted when a Song is going to be removed.
		@param s The Song
	*/
	void songGoingToBeRemoved(Song* s);
	
	/**
		Signal emitted when a directory is added.
		@param dir The directory
	*/
	void dirAdded(DBDir* dir); 
	
	/**
		Signal emitted when a directory is removed.
		@param dir The directory
	*/
	void dirRemoved(DBDir* dir); 
};

#endif
