/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@student.luc.ac.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qfile.h> 
//#include <arts/artsflow.h>
#include <kartsdispatcher.h> 
#include <kartsserver.h> 
#include <kplayobjectfactory.h>
#include <kplayobject.h>
#include <kaudiomanagerplay.h>
#include <kmessagebox.h>
#include <klocale.h>
#include "artsplayer.h"
#include "globals.h"


static QCString url_encode(const QString & str)
{
	return QFile::encodeName(str);
}

ArtsPlayer::ArtsPlayer(QObject *parent, const char *name ) 
	: PlayerBase(parent,name),
	m_dispatcher(0),m_server(0),m_factory(0),
	m_curr(0),m_aman_play(0), m_volume_ctrl(Arts::StereoVolumeControl::null()),
	m_paused(false),m_volume(1.0f)
{
	m_dispatcher = new KArtsDispatcher(this);
	m_server = new KArtsServer(this);
	setupArtsObjects();
	connect(m_server,SIGNAL(restartedServer()),this,SLOT(setupArtsObjects()));
}


ArtsPlayer::~ArtsPlayer()
{
	delete m_curr;
	delete m_factory;
	delete m_aman_play;
	delete m_server;
	delete m_dispatcher;
}

void ArtsPlayer::setupArtsObjects()
{
	delete m_factory;
    delete m_aman_play;
	m_volume_ctrl = Arts::StereoVolumeControl::null();
	m_factory = new KDE::PlayObjectFactory(m_server);
	m_aman_play = new KAudioManagerPlay(m_server);

	if (m_aman_play->isNull() || !m_factory) 
	{
		KMessageBox::error(0, i18n("Connecting/starting aRts soundserver failed. "
								"Make sure that artsd is configured properly."));
		exit(1);
	}

	m_aman_play->setTitle(i18n("MusicMan"));
	m_aman_play->setAutoRestoreID("MusicManAmanPlay");

	m_factory->setAudioManagerPlay(m_aman_play);
}

void ArtsPlayer::playObjectCreated()
{
	setVolume(m_volume);
}

int ArtsPlayer::totalTime() const
{
	if (m_curr)
	{
		Arts::poTime ct;
		ct = m_curr->overallTime();
		return ct.seconds;
	}
	else
		return 0;
}

int ArtsPlayer::currentTime() const
{
	if (isServerRunning() && m_curr && 
		(m_curr->state() == Arts::posPlaying || 
		m_curr->state() == Arts::posPaused))
	{
		return m_curr->currentTime().seconds;
	}
	else
		return 0;
}

bool ArtsPlayer::isIdle() const
{
	if (m_curr && m_curr->state() == Arts::posIdle)
		return true;
	else
		return false;
}

bool ArtsPlayer::isPlaying() const
{
	if (m_curr && m_curr->state() == Arts::posPlaying)
		return true;
	else
		return false;
}


bool ArtsPlayer::isServerRunning() const
{
	if(m_server)
		return !(m_server->server().isNull());
	else
		return false;
}

bool ArtsPlayer::play(const QString & file)
{
	if (m_curr)
	{
		delete m_curr;
		m_curr = 0;
	}
	
	m_curr = m_factory->createPlayObject(KURL(url_encode(file)),false);
	if (m_curr)
	{
		if(m_curr->object().isNull())
			connect(m_curr, SIGNAL(playObjectCreated()), SLOT(playObjectCreated()));
		else
			playObjectCreated();
		
		m_curr->play();
		m_paused = false;
	}
	
	return m_curr != 0;
}

bool ArtsPlayer::resume()
{
	if (m_curr)
	{
		m_paused = false;
		m_curr->play();
		return true;
	}
	else
	{
		return false;
	}
}

bool ArtsPlayer::pause()
{
	if (m_curr)
	{
		m_curr->pause();
		m_paused = true;
		return true;
	}
	else
	{
		return false;
	}
}

bool ArtsPlayer::stop()
{
	if (m_curr)
	{
		m_curr->halt();
		delete m_curr;
		m_curr = 0;
		m_paused = false;
		
		if(!m_volume_ctrl.isNull()) 
		{
			m_volume_ctrl.stop();
			m_volume_ctrl = Arts::StereoVolumeControl::null();
		}
		return true;
	}
	else
	{
		return false;
	}
}

void ArtsPlayer::seek(int secs)
{
	if (isServerRunning() && m_curr)
	{
		Arts::poTime pt;
		pt.custom = 0;
		pt.ms = 0;
		pt.seconds = secs;
		m_curr->object().seek(pt);
	}
}

float ArtsPlayer::setVolume(float vol)
{
	m_volume = vol;
	if (m_volume < 0)
		m_volume = 0;
	else if (m_volume > 1)
		m_volume = 1;
	
	if (isServerRunning() && m_curr && !m_curr->isNull()) 
	{
		
		if(m_volume_ctrl.isNull())
			setupVolumeControl();
		
		if(!m_volume_ctrl.isNull()) 
		{
			m_volume_ctrl.scaleFactor(m_volume);
		}
	
		//m_server->server().outVolume().scaleFactor(m_volume);
	}
	return m_volume;
}

void ArtsPlayer::setupVolumeControl()
{
	m_volume_ctrl = Arts::DynamicCast(m_server->server().createObject("Arts::StereoVolumeControl"));
	if(!m_volume_ctrl.isNull() && !m_curr->isNull() && !m_curr->object().isNull()) 
	{
		Arts::Synth_AMAN_PLAY ap = m_aman_play->amanPlay();
		Arts::PlayObject po = m_curr->object();
		ap.stop();
		Arts::disconnect(po, "left" , ap, "left" );
		Arts::disconnect(po, "right", ap, "right");

		m_volume_ctrl.start();
		ap.start();

		Arts::connect(po, "left" , m_volume_ctrl, "inleft" );
		Arts::connect(po, "right", m_volume_ctrl, "inright");
		Arts::connect(m_volume_ctrl, "outleft" , ap, "left" );
		Arts::connect(m_volume_ctrl, "outright", ap, "right");
	}
	else 
	{
		m_volume_ctrl = Arts::StereoVolumeControl::null();
	}
}

#include "artsplayer.moc"
