/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@student.luc.ac.be                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PLAYERBASE_H
#define PLAYERBASE_H

#include <qobject.h>

/**
@author Joris Guisson
@brief Base class for all players

This class is the base class for all music players. 
A music player provides the following functionality :
- plays media files
- pause / resume
- seeking in a music file
- status information
*/
class PlayerBase : public QObject
{
	Q_OBJECT
public:
	/**
	 * Constructor.
	 * @param parent The parent QObject
	 * @param name The name of the object
	 */
	PlayerBase(QObject *parent = 0, const char *name = 0);
	
	/**
	 * Destructor.
	 */
	virtual ~PlayerBase();
	
	/**
	 * Get the total time of the current song in seconds.
	 * @return The time in seconds of the current song
	 */
	virtual int totalTime() const = 0;
	
	/**
	 * Get the amount of time which has elapsed playing the
	 * current song.
	 * @return The time in seconds
	 */
	virtual int currentTime() const = 0;
	
	/**
	 * See if the player is idle (playing nothing).
	 * @return true when no song is playing
	 */
	virtual bool isIdle() const = 0;
	
	/**
	 * See if the player is paused.
	 * @return true if the player is paused
	 */
	virtual bool isPaused() const = 0;
	
	/**
	 * See if the player is playing.
	 * @return true if a song is playing
	 */
	virtual bool isPlaying() const = 0;
	
	/**
	 * Play a song.
	 * @param file Filename of the song
	 * @return true upon succes, false otherwise
	 */
	virtual bool play(const QString & file) = 0;
	
	/**
	 * Resume playing after the player was paused.
	 * This only works when the player is paused.
	 * @return true upon succes, false otherwise
	 */
	virtual bool resume() = 0;
	
	/**
	 * Pause playing, this only works if something is playing.
	 * @return true upon succes, false otherwise
	 */
	virtual bool pause() = 0;
	
	/**
	 * Stop playing, this only works if something is playing.
	 * @return true upon succes, false otherwise
	 */
	virtual bool stop() = 0;
	
	/**
	 * Play from somewhere in the middle of the song.
	 * @param secs The number of seconds into the song
	 */
	virtual void seek(int secs) = 0;
	
	/**
	 * Set the volume. The volume must be between 0 and 1, if it
	 * lies outside these values it will be clamped between 0 and 1 
	 * (i.e. < 0 becomes 0, > 1 becomes 1).
	 * @param vol The volume must lie between 0 and 1
	 * @return The volume which was set
	 */
	virtual float setVolume(float vol) = 0;

};

#endif
