/***************************************************************************
 *   Copyright (C) 2004 by Joris Guisson                                   *
 *   joris.guisson@pandora.be                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qdir.h> 
#include <kconfig.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kmessagebox.h>

#include "kmusicmanager.h"
#include "mainwindow.h"

#include "view/dbview.h"
#include "view/musiclistview.h"

#include "musicdb/song.h"
#include "musicdb/musicdatabase.h"
#include "player/musicplayer.h"
#include "player/playqueue.h"
#include "tools/toolmanager.h"
#include "util/error.h"
#include "globals.h"
#include "../libkmusicmanager/dialogs/dirmandlg.h"


//#include "actioncollection.h"

KMusicManager::KMusicManager()
	: KApplication(),m_first_boot(false)
{
	m_db = new MusicDatabase(this);
	m_player = new MusicPlayer(m_db,this);
	m_tool_man = new ToolManager(this);
}


KMusicManager::~KMusicManager()
{}

void KMusicManager::init()
{
	m_wnd = new MainWindow(this);
	setMainWidget(m_wnd);
	QString data_dir = getDataDir();
	try
	{
		OutputLog().init(data_dir + "outputlog");
	}
	catch (Error & err)
	{
		qDebug("Failed to create outputlog");
	}
		
	DBView* dbv = 0;
	if (!m_first_boot)
	{
		loadDB();
		m_tool_man->load();
		dbv = new DBView(m_db,m_wnd);
		m_wnd->addChildView(dbv,"collection");
	}
	else
	{
		m_tool_man->load();
		dbv = new DBView(m_db,m_wnd);
		m_wnd->addChildView(dbv,"collection");
		m_wnd->showChildView("collection");
	}
	
	
	loadSettings();
	
	m_player->getPlayQueue()->loadQueue(getDataDir(),m_db);
	m_player->setProvider(dbv->getMusicListView());
	OutputLog() << "=============================================" << endl;
}

QString KMusicManager::getDataDir()
{
	QString d = QDir::homeDirPath();
	if (!d.endsWith("/"))
		d += "/";
		
	d += ".kmusicmanager/";
	QDir dir(d);
	if (!dir.exists())
	{
		dir.mkdir(d);
		m_first_boot = true;
	}
	return d;
}

void KMusicManager::loadDB()
{
	try
	{
		m_db->loadDB(getDataDir() + "musicdb");
	}
	catch (Error & e)
	{
		OutputLog() << "Error : " << e.errorMessage() << endl;
		KMessageBox::error(m_wnd,"The database was corrupted !","Error",KMessageBox::Ok);
	}
}

void KMusicManager::saveDB()
{
	try
	{
		m_db->saveDB(getDataDir() + "musicdb");
	}
	catch (Error & e)
	{
		OutputLog() << "Error : " << e.errorMessage() << endl;
	}
}

void KMusicManager::saveSettings()
{
	KConfig* config = kapp->config();
	
	MusicListView::getViewConfig().saveSettings(config);
	m_player->saveSettings(config);
	m_wnd->saveCVSettings(config);
	config->setGroup("general");
	config->writeEntry("systrayicon",m_wnd->isSystemTrayIconEnabled());
	config->writeEntry("hidden",!m_wnd->isVisible());
	m_wnd->saveWindowSize(config);
	m_tool_man->saveSettings(config);
}

void KMusicManager::loadSettings()
{
	KConfig* config = kapp->config();
	
	MusicListView::getViewConfig().loadSettings(config);
	m_player->loadSettings(config);
	m_wnd->loadCVSettings(config);
	
	m_wnd->setActionStatus(m_player->getRandom(),m_player->getRepeat());
	
	config->setGroup("general");
	m_wnd->enableSysTrayIcon(config->readBoolEntry("systrayicon",false));
	
	if (config->readBoolEntry("hidden",false))
		m_wnd->hide();
	else
		m_wnd->show();
	
	m_wnd->restoreWindowSize(config);
	m_tool_man->loadSettings(config);
}

void KMusicManager::onExit()
{
	OutputLog() << "Saving DB and playlists" << endl;
	saveDB();
	m_player->getPlayQueue()->saveQueue(getDataDir());
	saveSettings();
	
	m_player->stop();
	m_tool_man->unload();

	OutputLog() << "Exit " << endl;
}

void KMusicManager::changePlayerSource(const QString & name,bool play_next)
{
	ChildView* cv = m_wnd->getChildView(name);
	if (cv)
	{
		m_player->setProvider(cv->getMusicListView());
		if (play_next)
			m_player->next();
	}
}

void KMusicManager::fileAddFiles()
{
	QStringList li = KFileDialog::getOpenFileNames(
			QString::null,
			"*.mp3 *.MP3 *.ogg *.OGG *.flac *.FLAC|Music Files\n*.*| All Files",
			m_wnd,"Select files to add");
	
	for (QStringList::iterator i = li.begin();i != li.end();i++)
	{
		Song* s = new Song(*i);
		try
		{
			s->loadInfoFromFile();
			m_db->addSong(s);
		}
		catch (Error & e)
		{
			OutputLog() << "Error : " << e.errorMessage() << endl;
			delete s;
		}
	}
}

void KMusicManager::fileAddDir()
{
	QString dir = KFileDialog::getExistingDirectory(
			QString::null,m_wnd,i18n("Select a directory"));
	
	if (dir != QString::null)
	{
		m_db->addDir(dir);
	}
}

void KMusicManager::fileManDirs()
{
	DirManDlg dlg(m_wnd);
	
	dlg.setMusicDataBase(m_db);
	dlg.show();
	dlg.exec();
}


#include "kmusicmanager.moc"
